﻿using EvaPOS_API_FRAME.DTO;
using EvaPosSrvDTO;
using EvaPosSrvResp;
using GatewaySCO;
using Serilog;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Sockets;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Threading;

namespace gatewayGK.POSBC
{
    ///<summary>
    /// Procesa solicitudes de ReportStatusEventsRequest.
    /// </summary>
    public class ReportStatusEventsRequestPosbcCmd : IComando
    {
        public string Referencia { get; set; } = "scsns:ReportStatusEvents";
        /// <summary>
        /// DTO con solicitud.
        /// </summary>
        public ReportStatusEventsRequestDTO Request { get; private set; }

        /// <summary>
        /// Procesa y responde ReportStatusEventsRequest.
        /// </summary>
        /// 
        public Respuestas Ejecutar()
        {
            string direccionIpPosbc = "127.0.0.1";
            int puerto = 6697;

            byte[] bufferRecibido = Entorno<EntornoPOSBC>.Instancia.get().BufferEntrada;
            int cantBytes = Entorno<EntornoPOSBC>.Instancia.get().CantBytes;
            try
            {
                // Crear un socket TCP/IP
                Socket socket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                // Configurar el endpoint (IP y puerto)
                IPEndPoint endPoint = new IPEndPoint(IPAddress.Parse(direccionIpPosbc), puerto);

                //Conectar al endpoint
                socket.Connect(endPoint);
                Log.Information("Conectandose al servidor POSBC");

                // Enviar los datos
                socket.Send(bufferRecibido, SocketFlags.None);
                Log.Information($"Buffer de entrada enviado al servidor POSBC {bufferRecibido}");
                // Configurar timeout para el socket (por ejemplo, 5 segundos)
                socket.ReceiveTimeout = 5000; // en milisegundos

                // Buffer para recibir la respuesta - Revisar cual es la mejor manera de recibir ese
                List<byte> receivedBytes = new List<byte>();
                byte[] bufferSalida = new byte[1024];


                while (true)
                {
                    int bytesLeidos = 0;

                    try
                    {
                        bytesLeidos = socket.Receive(bufferSalida, 0, bufferSalida.Length, SocketFlags.None);
                    }
                    catch (SocketException ex)
                    {
                        if (ex.SocketErrorCode == SocketError.TimedOut)
                        {
                            Log.Information("Tiempo de espera agotado. No se recibieron más datos.");
                            break;
                        }
                    }
                        //Coloque 1 Porque POSBC si no tiene mas bytes que mandar, manda 1 y no 0
                        if (bytesLeidos == 0 || bytesLeidos == 1)
                    {
                        // El socket se ha cerrado
                        break;
                    }

                    // Añadir los bytes leídos a la lista
                    for (int i = 0; i < bytesLeidos; i++)
                    {
                        receivedBytes.Add(bufferSalida[i]);
                    }
                }

                byte[] data = receivedBytes.ToArray();

                // Convertir el array de bytes a una cadena
                string result = Encoding.UTF8.GetString(data);
                Entorno<EntornoPOSBC>.Instancia.get().BufferSalida = data;

                socket.Shutdown(SocketShutdown.Both);
                socket.Close();
            }
            catch (SocketException ex)
            {
                Log.Error($"Error de Socket: {ex.Message}");
                Log.Error($"Código de error de Socket: {ex.SocketErrorCode}");
                Log.Error(ex.StackTrace);
            }
            catch (Exception ex)
            {
                Log.Error("Error:", ex.Message);
            }
            //Esto como se manejara? 
            return new Respuestas();
        }

        public IComando CreaCopia()
        { 
          return (ReportStatusEventsRequestPosbcCmd)this.MemberwiseClone();
        }
        public void CargaDTO(DTOBase reportStatusEventsRequestDTO)
        {
            Request = (ReportStatusEventsRequestDTO)reportStatusEventsRequestDTO;
        }
    }
}
